@@ -20,9 +20,14 @@ along with OpenSesame.  If not, see <http://www.gnu.org/licenses/>.
 from libopensesame.py3compat import *
 import os
 import sys
+import time
 import mimetypes
 import textwrap
 import inspect
+try:
+    from collections import OrderedDict
+except ImportError:
+    OrderDict = dict
 from libopensesame import metadata
 from libopensesame.oslogging import oslogger
 from libqtopensesame.extensions import BaseExtension
@@ -89,7 +94,7 @@ class OpenSesameIDE(BaseExtension):
         self._scetw.default_extension = \
             lambda: cfg.opensesame_ide_default_extension
         self._add_ide_tab()
-        self._dock_widgets = {}
+        self._dock_widgets = OrderedDict()
         self._set_ignore_patterns()
         self._restore_open_folders()
         self._parse_command_line()
@@ -308,10 +313,6 @@ class OpenSesameIDE(BaseExtension):
             clean_trailing_whitespaces=cfg.opensesame_ide_strip_lines,
             show_whitespaces=None
         )
-        self.extension_manager.fire(
-            u'register_editor',
-            editor=editor
-        )
 
     def remove_folder_browser_dock_widget(self, dock_widget):
 
@@ -470,6 +471,16 @@ class OpenSesameIDE(BaseExtension):
         oslogger.debug(
             'setting folder-browser visibility to {}'.format(hidden)
         )
+        # This ensures that, when multiple dockwidgets are tabified together,
+        # the correct one is on top when the dockwidgets are revealed again.
+        # This is done by moving the invisible dockwidgets to the end of the
+        # dict of dockwidgets so that they are made visible last. This only
+        # works for recent versions of Python that support the
+        # OrderDict.move_to_end() function.
+        if not hidden and hasattr(self._dock_widgets, 'move_to_end'):
+            for path, dockwidget in list(self._dock_widgets.items()):
+                if dockwidget.visibleRegion().isEmpty():
+                    self._dock_widgets.move_to_end(path)
         for dockwidget in self.dock_widgets:
             dockwidget.setVisible(hidden)
 
@@ -572,6 +583,8 @@ class OpenSesameIDE(BaseExtension):
             u'jupyter_change_dir',
             path=os.path.dirname(path)
         )
+        time.sleep(.1)
+        QApplication.processEvents()
     
     @with_editor_and_cursor
     def run_from_current_position(self, editor, cursor):
@@ -969,7 +982,7 @@ class OpenSesameIDE(BaseExtension):
         tw.tab_closed.connect(self._on_editor_close)
         tw.split_requested.disconnect(splitter.split)
         tw.split_requested.connect(self._on_split_requested)
-        splitter.editor_created.connect(self._on_editor_created)
+        splitter.document_opened.connect(self._on_editor_created)
         
     def _on_split_requested(self, widget, orientation, index=None):
         
